// Arduino base scale light controller for RC applications
// Written by Scott White, drzoo2@yahoo.com, January 03 2015
// Revision 001 Modified from Scale Light Controller project Revision 005 for Wraith Project. Includes looking at Channel 3 throttle position
              //to perform other light functions.
// Revision 002 Released. Added police, hazard and off functions to be controlled via channel3
// Revision 003 Added ability to calibrate endpoints for where scale blinkers turn on. Now writing to two more memory locations.
              // Fixed a bug in reverse steering that caused the brake lights to flash when kINTstp = 0

// User adjustable Assignments----------------------------------------------------------
const int kINTstp = 1;       // 0 = Just brakelights, no blinky. 1 = Declares Stoplights into Blinky. Pin setups stay the same
const int kFlash = 100;      // Blinker Speed in (mS). May affect timing to some degree.
const int kBRTblk = 50;      // Blinker Running Brightness. Only use small changes. Default 50
const int kBRThl = 25;       // Headlight Running Brightness. Only use small changes. Default 25
const int kBRTbrk = 30;      // Brakelight Running Brightness. Only use small changes. Default 30
//----------------------------------------------------------------------------------------

// Do not touch these! Unless you know what you are doing
const int kRSVCH3 = 15;       // Setup aux pin port (FUTURE)
const int kRSVCH1 = 16;       // Setup Steering pin port
const int kRSVCH2 = 14;       // Setup Throttle pin port
const int kRRLED = 10;        // Right Rear Stop LED
const int kLRLED = 9;         // Left Rear Stop LED
const int kRFLED = 3;         // Righ Blinker LED
const int kLFLED = 5;         // Left Blinker LED
const int kHLLED = 6;         // Headlight LEDs
const int kFOGLED = 8;        // FOG Headlight LEDs
const int kFlasher = 25;      // cal led flash speed

unsigned long iVal;          // General unsigned Variable
unsigned long iValCH1;       // Channel 1 READ Variable
unsigned long iValCH2;       // Channel 2 READ Variable
unsigned long iValCH3;       // Channel 3 READ Variable
unsigned long iValCALST;     // Cal Steering General Read Variable
unsigned long iValCALSTR;    // Calibrated Deadzone Channel 1 255
unsigned long iValCALSTL;    // Calibrated Deadzone Channel 1 0
unsigned long iValCALTHR;    // Cal Throttle General Read Variable
unsigned long iValCALFWD;    // Calibrated Deadzone Channel 2 255
unsigned long iValCALRVS;    // Calibrated Deadzone Channel 2 0

int kCali = 1;               // Calibrate on restart
int kPROMST = 0;             //EEPROM Variable for Steering position
int kPROMDRV = 0;            //EEPROM Variable for Driving position
int kPROMBLR = 0;            //EEPROM Variable for Steering Blink Time Right
int kPROMBLL = 0;            //EEPROM Variable for Steering Blink Time Left

#include <EEPROM.h>

void setup (){
  // Pin Functions
  //pinMode(kRRLED, OUTPUT);  //NOW A PWM OUTPUT
  //pinMode(kLRLED, OUTPUT);  //NOW A PWM OUTPUT
  //pinMode(kRFLED, OUTPUT);  //NOW A PWM OUTPUT
  //pinMode(kLFLED, OUTPUT);  //NOW A PWM OUTPUT
  //pinMode(kHLLED, OUTPUT);  //NOW A PWM OUTPUT
  pinMode(kFOGLED, OUTPUT);
  pinMode(kRSVCH1, INPUT);  //PIN16
  pinMode(kRSVCH2, INPUT);  //PIN14
  pinMode(kRSVCH3, INPUT);  //Pin15
}



// Calibration Function
// This code can be called at power on by holding both throttle and steering, any position
void calibration(){
  analogWrite(kRRLED, 0); //Set LED states to off
  analogWrite(kLRLED, 0);
  analogWrite(kRFLED, 0);
  analogWrite(kLFLED, 0);
  analogWrite(kHLLED, 0);
  delay (5000);   // Delay enshures user can change forward/reverse on remote after power on
  // Look for special input to change to reversed channel on remote
  // if value is around 1.5mS skip. Needs to be wide, close as possible to 1mS or 2mS
  //  else look to see which direction is triggered
  // store value in RAM and so appropriate function runs
  // EEPROM.write(address, value);
  
  iValCH1 = pulseIn(kRSVCH1, HIGH, 100000);
  if (iValCH1 < 1250 || iValCH1 > 1750){    // Look to see if user is holding steering posistion for calibration mode
     //If TRUE, Start Channel Calibrations mode, else go center calibrations
     
    //This is Programming mode. Loop until completed
    //Check if user has released mode wait until sure
    iVal = 0;
    while(iVal == 0){    //Wait here until user releases remote
      iValCH1 = pulseIn(kRSVCH1, HIGH, 100000); 
      analogWrite(kRRLED, 255);  //Flash all LEDS to incate in cal mode.
      analogWrite(kLRLED, 255);
      analogWrite(kRFLED, 255);
      analogWrite(kLFLED, 255);
      analogWrite(kHLLED, 255);
      delay(50);
      analogWrite(kRRLED, 0); 
      analogWrite(kLRLED, 0);
      analogWrite(kRFLED, 0);
      analogWrite(kLFLED, 0);
      analogWrite(kHLLED, 0);
      delay(50);
      
      if (iValCH1 > 1250 && iValCH1 < 1750){    // Look if user is still holding steering posistion for calibration mode. Continue when false
        iVal = 1;
        analogWrite(kRRLED, 0); 
        analogWrite(kLRLED, 0);
        analogWrite(kRFLED, 0);
        analogWrite(kLFLED, 0);
        analogWrite(kHLLED, 0);
        delay(500);
      } 
    }
    
    //Turn on Blinky to indicate Steering calibration
    analogWrite(kRFLED, 255);
    analogWrite(kLFLED, 255);
    iVal = 0;
    while(iVal == 0){  //Wait here until user makes a steering change      
      iValCH1 = pulseIn(kRSVCH1, HIGH, 100000); // Look to see if user is holding steering posistion for channel setup
      if (iValCH1 > 1600){ //channel read forward
        EEPROM.write(0, 0); //steering address 0
        iVal = 1;
      }
      if (iValCH1 < 1400){ //channel read reversed
        EEPROM.write(0, 1); //steering address 0
        iVal = 1;
      }      
    }
    //Turn off Blinky to indicate steering complete
    analogWrite(kRFLED, 0);
    analogWrite(kLFLED, 0);
    delay(1000);
 
    //Turn on HL and Stop light to indicate driving calibration
    analogWrite(kRRLED, 255); 
    analogWrite(kLRLED, 255);
    analogWrite(kHLLED, 255);
    iVal = 0;
    while(iVal == 0){
      iValCH2 = pulseIn(kRSVCH2, HIGH, 100000);  // Look to see if user is holding throttle posistion for channel setup
      if (iValCH2 < 1350){ //channel read forward
      EEPROM.write(1, 0);  //driving address 1
      iVal = 1;
      }
    if (iValCH2 > 1650){ //channel read reversed
      EEPROM.write(1, 1);  //driving address 1
      iVal = 1;
      }      
    }
    //Turn off HL/Stop lights to indicate throttle complete
    analogWrite(kHLLED, 0);
    analogWrite(kRRLED, 0);
    analogWrite(kLRLED, 0);
    
    // Boundry scan to make sure positions are close to center and user is not hold one or the other accidently. 
    iVal = 0; 
    while (iVal == 0){
      iValCH1 = pulseIn(kRSVCH1, HIGH, 100000); 
      iValCH2 = pulseIn(kRSVCH2, HIGH, 100000);  
      if (iValCH1 < 1400 || iValCH1 > 1600 || iValCH2 < 1400 || iValCH2 > 1600){    // Look to see if user is holding any posistion
        iVal = 0;
      }
      else{
        iVal = 1;
      }
    }
    delay(1000);
    
    // Calibrate Steering blinker "ON" percentage
    iVal = 0;
    while(iVal == 0){
      analogWrite(kRFLED, 255);
      iValCH2 = pulseIn(kRSVCH2, HIGH, 100000);  // Look to see if user is holding throttle posistion for channel setup
      iValCH1 = pulseIn(kRSVCH1, HIGH, 100000);
      if (iValCH2 < 1400 || iValCH2 > 1600){ //channel 1 read for indication
        iValCH1 = iValCH1 / 10;
        iValCH1 = round(iValCH1);
        EEPROM.write(2, iValCH1);  //LED RIGHT BLINK address 2
        iVal = 1;
        analogWrite(kRFLED, 0);
      }     
    }
    
    // Boundry scan to make sure positions are close to center and user is not hold one or the other accidently. 
    iVal = 0; 
    while (iVal == 0){
      iValCH1 = pulseIn(kRSVCH1, HIGH, 100000); 
      iValCH2 = pulseIn(kRSVCH2, HIGH, 100000);  
      if (iValCH1 < 1400 || iValCH1 > 1600 || iValCH2 < 1400 || iValCH2 > 1600){    // Look to see if user is holding any posistion
        iVal = 0;
      }
      else{
        iVal = 1;
      }
    }
    delay(1000);
  
    iVal = 0;
    while(iVal == 0){
      analogWrite(kLFLED, 255);
      iValCH2 = pulseIn(kRSVCH2, HIGH, 100000);  // Look to see if user is holding throttle posistion for channel setup
      iValCH1 = pulseIn(kRSVCH1, HIGH, 100000);
      if (iValCH2 < 1400 || iValCH2 > 1600){ //channel 1 read for indication
        iValCH1 = iValCH1 / 10;
        iValCH1 = round(iValCH1);
        EEPROM.write(3, iValCH1);  //LED RIGHT BLINK address 2
        iVal = 1;
        analogWrite(kLFLED, 0);
        }     
    }  
  }
  
  // Boundry scan to make sure positions are close to center and user is not hold one or the other accidently. 
  iVal = 0; 
  while (iVal == 0){
    iValCH1 = pulseIn(kRSVCH1, HIGH, 100000); 
    iValCH2 = pulseIn(kRSVCH2, HIGH, 100000);  
    if (iValCH1 < 1400 || iValCH1 > 1600 || iValCH2 < 1400 || iValCH2 > 1600){    // Look to see if user is holding any posistion
      iVal = 0;
    }
    else{
      iVal = 1;
    }
  }
  delay(1000);
  
  //This code always runs once at power on
  delay(500);
  iValCALST = 0;
  iValCALTHR = 0;
  for(int x = 0; x < 5; x++){  
    iValCH1 = pulseIn(kRSVCH1, HIGH, 100000); // Read steering posistion
    iValCALST = iValCH1 + iValCALST;
      
    iValCH2 = pulseIn(kRSVCH2, HIGH, 100000);  // Read throttle posistion
    iValCALTHR = iValCH2 + iValCALTHR;  
        
    analogWrite(kRRLED, 255);  // Pretty Flashing
    delay (kFlasher);
    analogWrite(kRRLED, 0);
    delay (kFlasher);    
    analogWrite(kLRLED, 255);
    delay (kFlasher);
    analogWrite(kLRLED, 0);
    delay (kFlasher);
    analogWrite(kLFLED, 255);
    delay (kFlasher);
    analogWrite(kRFLED, 0);
    delay (kFlasher);
    analogWrite(kHLLED, 255);
    delay (kFlasher);
    analogWrite(kHLLED, 0);
    delay (kFlasher);
    analogWrite(kRFLED, 255);
    delay (kFlasher);
    analogWrite(kLFLED, 0);
    delay (kFlasher);
  }
  
  // Look at flag in RAM to call appropriate funtion. Set RAM if any other value other than 0 or 1
  kPROMST = EEPROM.read(0);  //read steering address data
  kPROMDRV = EEPROM.read(1);  //read driving address data
  
  if (kPROMST == 0){          // If = 0, Channel is normal
  kPROMBLR = EEPROM.read(2);
  kPROMBLL = EEPROM.read(3);
  }
  if (kPROMST == 1){          // If = 1, Channel is reversed
  kPROMBLL = EEPROM.read(2);
  kPROMBLR = EEPROM.read(3);
  }
  kPROMBLR = kPROMBLR * 10;
  kPROMBLL = kPROMBLL * 10;
  
  iValCALST = iValCALST / 5;        // Take average steering posistion
  iValCALSTR = kPROMBLR;    // Set Deadzone HIGH
  iValCALSTL = kPROMBLL;     // Set Deadzone LOW
    
  iValCALTHR = iValCALTHR / 5;      // Take average throttle posistion
  iValCALFWD = iValCALTHR * 1.02;    // Set Deadzone LOW
  iValCALRVS = iValCALTHR * 0.99;    // Set Deadzone HIGH
 
  for(int x = 0; x < 10; x++){      // More Pretty Flashing
    analogWrite(kRRLED, 255);
    analogWrite(kLRLED, 255);
    analogWrite(kLFLED, 255);
    analogWrite(kRFLED, 255);
    analogWrite(kHLLED, 255);
    delay (50);
    analogWrite(kRFLED, 0);
    analogWrite(kLFLED, 0);
    analogWrite(kRRLED, 0);
    analogWrite(kLRLED, 0);
    analogWrite(kHLLED, 0);
    delay (50);
  }
}



// Steering Blinker Function
// When called
void signalsST(){
  //Steering normal channel 1
  if (kPROMST == 0){
    if (iValCH1 > iValCALSTR){
      if (kINTstp == 1){
        analogWrite(kRRLED, 0);
      }
      analogWrite(kRFLED, 0);
      delay (kFlash);
      if (kINTstp == 1){
        analogWrite(kRRLED, 255);
      }
      analogWrite(kRFLED, 255);
      delay (kFlash);
      if (kINTstp == 1){
        analogWrite(kRRLED, kBRTbrk);
      }
      analogWrite(kRFLED, kBRTblk);
    }
    if (iValCH1 < iValCALSTL){
      if (kINTstp == 1){      
        analogWrite(kLRLED, 0);
      }
      analogWrite(kLFLED, 0);
      delay (kFlash);
      if (kINTstp == 1){      
        analogWrite(kLRLED, 255);
      }
      analogWrite(kLFLED, 255);
      delay (kFlash);
      if (kINTstp == 1){     
        analogWrite(kLRLED, kBRTbrk);
      }
      analogWrite(kLFLED, kBRTblk);
    }
  }
  //Steering reverse channel 1
  if (kPROMST == 1){
    if (iValCH1 < iValCALSTL){
      analogWrite(kRFLED, 0);
      if (kINTstp == 1){      
        analogWrite(kRRLED, 0);
      }
      delay (kFlash);
      if (kINTstp == 1){      
        analogWrite(kRRLED, 255);
      }
      analogWrite(kRFLED, 255);
      delay (kFlash);
      if (kINTstp == 1){
        analogWrite(kRRLED, kBRTbrk);
      }
      analogWrite(kRFLED, kBRTblk);
    }
    if (iValCH1 > iValCALSTR){
      analogWrite(kLFLED, 0);
      if (kINTstp == 1){
        analogWrite(kLRLED, 0);
      }
      delay (kFlash);
      if (kINTstp == 1){
        analogWrite(kLRLED, 255);
      }
      analogWrite(kLFLED, 255);
      delay (kFlash);
      if (kINTstp == 1){
        analogWrite(kLRLED, kBRTbrk);
      }
      analogWrite(kLFLED, kBRTblk);   
    }
  }    
}



// Brakelight Function
void signalsTHR(){ 
  //Brake Lights normal channel 2
  if (kPROMDRV == 0){
    while (iValCH2 < iValCALRVS){     // While loop used to maintain solid light
      analogWrite(kRRLED, 255);
      analogWrite(kLRLED, 255);
      iValCH2 = pulseIn(kRSVCH2, HIGH, 100000);
       
      iValCH1 = pulseIn(kRSVCH1, HIGH);                  // Check for Blinker Function. This is a loop and would be ignores otherwise
      if (iValCH1 > iValCALSTR || iVal < iValCALSTL){
        signalsST();
      }
    }
    //Head Lights normal channel 2
    while (iValCH2 > iValCALFWD){     // While loop used to maintain solid light
      analogWrite(kHLLED, 255);
      analogWrite(kRRLED, kBRTbrk);
      analogWrite(kLRLED, kBRTbrk);
      iValCH2 = pulseIn(kRSVCH2, HIGH);
        
      iValCH1 = pulseIn(kRSVCH1, HIGH);                  // Check for Blinker Function. This is a loop and would be ignores otherwise
      if (iValCH1 > iValCALSTR || iVal < iValCALSTL){
        signalsST();
      }
    }
  } 
    //Brake Lights reverse channel 2
  if (kPROMDRV == 1){
    while (iValCH2 > iValCALFWD){     // While loop used to maintain solid light
      analogWrite(kRRLED, 255);
      analogWrite(kLRLED, 255);
      iValCH2 = pulseIn(kRSVCH2, HIGH, 100000);
       
      iValCH1 = pulseIn(kRSVCH1, HIGH);                  // Check for Blinker Function. This is a loop and would be ignores otherwise
      if (iValCH1 > iValCALSTR || iVal < iValCALSTL){
        signalsST();
      }
    }
    //Head Lights reverse channel 2
    while (iValCH2 < iValCALRVS){     // While loop used to maintain solid light
      analogWrite(kHLLED, 255);
      analogWrite(kRRLED, kBRTbrk);
      analogWrite(kLRLED, kBRTbrk);      
      iValCH2 = pulseIn(kRSVCH2, HIGH);
        
      iValCH1 = pulseIn(kRSVCH1, HIGH);                  // Check for Blinker Function. This is a loop and would be ignores otherwise
      if (iValCH1 > iValCALSTR || iVal < iValCALSTL){
        signalsST();
      }
    }
  } 
}



void loop (){
  if (kCali == 1){  // Call calibration function, run only once on startup
    kCali = 0;
    calibration();
  }
   
  //This function turns LEDs OFF
  iValCH3 = pulseIn(kRSVCH3, HIGH);
  if (iValCH3 < 1165){ //mid value is 1ms
    // Set Running LED Brightness
    analogWrite(kRRLED, 0);
    analogWrite(kLRLED, 0);
    analogWrite(kRFLED, 0);
    analogWrite(kLFLED, 0);
    analogWrite(kHLLED, 0);
    digitalWrite(kFOGLED, LOW);
  }
  //This function turns LEDs on for Scale operation.
  if (iValCH3 > 1165 && iValCH3 < 1495){ //mid value is 1.33ms
    analogWrite(kRRLED, kBRTbrk);
    analogWrite(kLRLED, kBRTbrk);
    analogWrite(kRFLED, kBRTblk);
    analogWrite(kLFLED, kBRTblk);
    analogWrite(kHLLED, kBRThl);
    digitalWrite(kFOGLED, HIGH);
    iValCH1 = pulseIn(kRSVCH1, HIGH); // Watch reciever CH1 for changes.
    if (iValCH1 > iValCALSTR || iValCH1 < iValCALSTL){
      signalsST();
    }
    iValCH2 = pulseIn(kRSVCH2, HIGH); // Watch reciever CH2 for changes.
    if (iValCH2 > iValCALRVS || iValCH2 < iValCALFWD){
      signalsTHR();
    }
  }
  //This function turns LEDs on for Hazard operation.
  if (iValCH3 > 1495 && iValCH3 < 1825){ //mid value is 1.66ms
    analogWrite(kRRLED, 255);
    analogWrite(kLRLED, 255);
    analogWrite(kRFLED, 255);
    analogWrite(kLFLED, 255);
    analogWrite(kHLLED, 255);
    digitalWrite(kFOGLED, HIGH);
    delay(400);
    analogWrite(kRRLED, 0);
    analogWrite(kLRLED, 0);
    analogWrite(kRFLED, 0);
    analogWrite(kLFLED, 0);
    delay(400);
  }
  //This function turns LEDs on for Spacy operation.
  if (iValCH3 > 1825){ //mid value is 2ms
    analogWrite(kRRLED, 255);
    analogWrite(kLRLED, 255);
    delay(40);
    analogWrite(kRRLED, 0);
    analogWrite(kLRLED, 0);
    delay(40);
    analogWrite(kRRLED, 255);
    analogWrite(kLRLED, 255);
    delay(40);
    analogWrite(kRRLED, 0);
    analogWrite(kLRLED, 0);
    delay(40);
    analogWrite(kRRLED, 255);
    analogWrite(kLRLED, 255);
    delay(40);
    analogWrite(kRRLED, 0);
    analogWrite(kLRLED, 0);
    delay(40);
    analogWrite(kRFLED, 255);
    analogWrite(kLFLED, 255);
    delay(40);
    analogWrite(kRFLED, 0);
    analogWrite(kLFLED, 0);
    delay(40);
    analogWrite(kHLLED, 255);
    delay(40);
    analogWrite(kHLLED, 0);
    delay(40);
    analogWrite(kHLLED, 255);
    delay(40);
    analogWrite(kHLLED, 0);
    delay(40);
    analogWrite(kHLLED, 255);
    delay(40);
    analogWrite(kHLLED, 0);
    delay(40);
    digitalWrite(kFOGLED, HIGH);
    delay(40);
    digitalWrite(kFOGLED, LOW);
    delay(40);
    digitalWrite(kFOGLED, HIGH);
    delay(40);
    digitalWrite(kFOGLED, LOW);
    delay(40);
    digitalWrite(kFOGLED, HIGH);
    delay(40);
    digitalWrite(kFOGLED, LOW);
    delay(40);
  }
}
